#!/usr/bin/env python

################################################################################
#
# FIESscript
#
# FIESscript.py
#
# Last update 20/05/2008
#
################################################################################


from configobj import *
import textLog
import messageLog
import config
import tasks
import pipeLine
import sys
import os
import getopt


# Catch calls to logging routines and move these to the text-based logfile

messageLog.put = textLog.put
messageLog.put_warning = textLog.put_warning
messageLog.put_error = textLog.put_error

# Turn of the interactive graphics of the following routines
# (not a very nice hack, but it works fine...
tasks.findord.interactive           = 'no'
tasks.findinterlacedord.interactive = 'no'
tasks.wavecal.interactive           = 'no'

# Set up some defaults values

cmdcurrentmode = 'QuickLook'
textconfigfile = 'textdefault.cfg'
singletask     = None

# Get the command line arguments
try:
  cmdoptions, infiles = getopt.getopt(sys.argv[1:],
                             "c:m:t:", ["configfile=", "mode=", "task="],
			     )

# And print out some help if it does not work...
except getopt.GetoptError:
  # print help information and exit:
  print "Unknown command line parameter. Allowed optional parameters are : "
  print "-c [file]        Use configuration 'file'"
  print "-m [mode]        Use reduction 'mode'"
  print "-t [task]        Perform only 'task' (calibrations only)"
  sys.exit(1)


# Display a welcome message in the message log
messageLog.put('='*55)
messageLog.put("This is FIESscript - the scriptable version of FIEStool")
messageLog.put('='*55)


# Process command line options
for (option, value) in cmdoptions:

  if option in ("-t", "--task"):
    singletask = value
  if option in ("-m", "--mode"):
    cmdcurrentmode = value
  if option in ("-c", "--configfile"):
    textconfigfile = value

# Check for error?
if not singletask and not infiles: 
  messageLog.put_error("One or more filenames should be given")
  sys.exit(1)

# And for another error?
if not os.path.exists(textconfigfile):
  messageLog.put_error("You have to specify a valid configuration file name.")
  messageLog.put_error("Configuration file %s does not exist" % textconfigfile)
  sys.exit(1)



# OK... config seems OK, now starting the pipeline

# First, read specified configuration file
config.load(textconfigfile)

# Check for error
if not cmdcurrentmode in config.options['reductionmodes'].value:
  messageLog.put_error("Requested reduction mode '%s' does not exist" % cmdcurrentmode)
  errstr = "Choose from :"
  for a_mode in config.options['reductionmodes'].value.keys():
    errstr += " %s" % a_mode
  messageLog.put_error(errstr)
  sys.exit(1)

# Create an instance of the pipeLine
pipeLine = pipeLine.pipeLine()

# If only one single (calibration) task was requested, run it!
if singletask:

  # Set up 'mode' environment parameter to 'CalibCalc'
  config.options['currentmode'].value = 'CalibCalc'
  pipeLine.run_singletask(singletask)
  sys.exit(0)



# Otherwise, run a full automated reduction of one or more frames

# Get the current reduction mode and prepare the list of tasks to perform
if cmdcurrentmode:
  config.options['currentmode'].value = cmdcurrentmode
currentmode = config.options['currentmode'].value

# Setup the pipeline to the selected reduction mode
errstr = pipeLine.set_mode(currentmode)
if errstr:
  messageLog.put_error(errstr)
  sys.exit(1)


# Start the pipeline, looping over given filenames
for infile in infiles:
  if not os.path.dirname(infile):
    infile = os.path.join(config.options['inpath'].value, infile)
  if not os.path.exists(infile):
    messageLog.put_error("File %s does not exist" % infile)
    sys.exit(1)

  # Here's the magic...
  pipeLine.process_frame(infile)

